/* Comment out this statement to print debugging messages to the
   stderr stream. This statement effects this file only */
#define NDEBUG 

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

/* Initilize the run */
#include "hazard.h"
#include <stmtopts.h>
#include <xlmode.h>
#include <hzf_log1.h>
#include <hzfxit.h>

#include <yyparse.h>
#include <stdlib.h>
#include <string.h>

/* #define NDEBUG */
/****************************************************************/

void initprz(void){

  /**
     VERSION is defined in the config.h file generated by the
     configure script.
  */

  char version[128];
  char version_text[128] = "Note: Procedure HAZARD (C-Version ";
  sprintf(version, "%s)",VERSION);

  /*
    yyparse parses the input command file (.sas) supplied to the 
    stdin at program execution. 
  */
  
#ifndef NDEBUG
  fprintf(stderr, "Parse input instructions\n");
#endif

  yyparse();
  
#ifndef NDEBUG
  fprintf(stderr, "Checking for parse errors\n");
#endif

  stmtptr = anchor;
  C->errorno = 0;
  if(!yysynerr) {
    /*  I would realy like to enum these stmtopts parameters to 
	make them human legible.  */
    if(stmtopts(40))
      xlmode(0);
    if(stmtopts(41))
      xlmode(1);
  }

  
#ifndef NDEBUG
  /* The same as debug statements for the PC version only */
  fprintf(stderr, "Dump run headers\n");
#endif
  /* Dump a run header */
  hzf_log1(" ");
  
  hzf_log1(strcat(version_text, version));
#ifdef HAZOPTIM
  hzf_log1("      Optimized version");
#endif
#ifndef NDEBUG
  hzf_log1("      DEBUGGING VERSION");
#endif /* NDEBUG */
  hzf_log1("      The Cleveland Clinic Foundation");
  hzf_log1("      9500 Euclid Avenue");
  hzf_log1("      Cleveland, Ohio 44195");
  hzf_log1("      Phone: (216) 444-6712");
  hzf_log1("      Email: hazard@bio.ri.ccf.org");
  
  /* For errors parsing the input command file, stop here 
     with a message */
  if(yysynerr) {
    hzf_log1("Note: PROCEDURE TERMINATING DUE TO ABOVE ERRORS.");
    hzfxit("SYNTAX");
  }
}
