#include <common.h>
#include <hzr_set_error.h>
#include <hzd_theta_to_parms.h>

void RESTOR(double *beta,long nvar,long *index,double *theta)

     /***********************************************************************

  RESTORE ESTIMATION VECTOR AND MODEL PARAMETERS

   PURPOSE
     IF NUMERICAL DERIVATIVES HAVE BEEN USED, THE ESTIMATION VECTOR
     THETA HAS BEEN JITTERED SLIGHTLY, AND THUS IT AND THE MODEL
     PARAMETERS ARE OFF THE OPTIMIZED VALUES A BIT.  THIS ROUTINE
     RESTORES THETA FROM BETA, AND THEN RESTORES THE MODEL PARAMETERS.

   DESCRIPTION OF PARAMETERS
     INPUT
       BETA   - REAL*8  NVAR-VECTOR OF CORRECT ESTIMATES
       NVAR   - INTEGER LENGTH OF BETA VECTOR
       INDEX  - INTEGER NTHETA-VECTOR POINTERS TO ACTIVE PARAMETERS
                        CORRESPONDING EXACTLY TO BETA, AND TELLING US
                        WHERE IN THETA TO PUT THE BETA.
       STATUS - INTEGER NTHETA-VECTOR OF MODEL STATUS INDICATORS

     OUTPUT
       THETA  - REAL*8  NTHETA-VECTOR OF ESTIMATES (JITTERING REMOVED)
       ERROR  - INTEGER ERROR FLAG OUTPUT TO COMMON /ERROR/
                        0 - NO ERROR
                        1 - TERMINAL ARITHMETIC ERROR
                        2 - TERMINAL DATA INPUT ERROR
       ERRFLG - CHARACTER*12 ERROR MESSAGE

   REMARKS
     1.  BETA IS NOT ALTERED BY THE SUBPROGRAM

     2.  THIS ROUTINE SHOULD BE CALLED AFTER THE COMPLETION OF THE
         OPTIMIZATION IF NUMERICAL DERIVATIVES HAVE BEEN USED RATHER
         THAN ANALYTICAL ONES.  IT SHOULD ALSO BE USED AFTER
         CALCULATION OF THE COVARIANCE MATRIX IF NUMERICAL DERIVATIVES
         ARE USED.

   SUBPROGRAMS REQUIRED
     DTRSFM - INTERCONVERT ESTIMATION AND MODEL PARAMETERS

   METHOD
     THETA IS RESTORED BY INSERTING VALUES OF BETA AS INDICATED BY
     INDEX.  THEN THE ROUTINE THAT TRANSFORMS THETA TO MODEL PARAMETERS
     IS INVOKED TO RESTORE THESE AS WELL.

   IMPLEMENTATION
     E.H. BLACKSTONE, MD     APRIL, 1986

   REVISIONS
     NONE

   VERIFICATION
     APRIL 24,1986
     THE RECALCULATED OBJECTIVE FUNCTION WAS SHOWN TO EXACTLY MATCH
     THAT ON THE FINAL ITERATION.

***********************************************************************/

{
  long i,j;

  /* INITIALIZE */
  if(nvar<=0) {
    hzr_set_error("(RESTOR900)",2);
    return;
  }
  Common.errorno = 0;
  CLEAR(Common.errflg);


  /* RETRIEVE THETAS FROM CURRENT PARAMETER VALUE VECTOR */
  for(i=0; i<nvar; i++) {
    j = index[i];
    theta[j] = beta[i];
  }

  /* NOW TRANSFORM THETA TO MODEL PARAMETERS */
  hzd_theta_to_parms(TRUE,TRUE);
}
